import 'dart:async';
import 'dart:io' as which_platform;
import 'package:flutter/foundation.dart';
import 'package:url_launcher/url_launcher.dart';
import '../../../hera/hera.dart';

extension HeraExtenstion on Hera {
  static Future<void> redirect(Uri authorizationUrl) async {
    if (await canLaunchUrl(authorizationUrl)) {
      await launchUrl(authorizationUrl, mode: LaunchMode.platformDefault, webOnlyWindowName: "_self");
    }
  }

  /// Determine the name of the device.
  static Future<String> getDeviceInfo() async {
    try {
      if (kIsWeb) {
        return "Web";
      } else {
        if (which_platform.Platform.isAndroid) {
          return "Android";
        } else if (which_platform.Platform.isIOS) {
          return "iOS";
        } else if (which_platform.Platform.isLinux) {
          return "Linux";
        } else if (which_platform.Platform.isMacOS) {
          return "MacOS";
        } else if (which_platform.Platform.isWindows) {
          return "Windows";
        }
      }
    } catch (e) {
      print("could not get device info with err: " + e.toString());
    }
    return "";
  }

  static Stream<T> imageStream<T>(
      Uint8List data,
      T req,
      T Function(List<int>) f,
      ) async* {
    if (data.lengthInBytes > 100 * 100000) throw 'too large';
    yield req;
    Iterator<int> it = data.iterator;
    for (var _ = 0; _ < 100; _++) {
      while (true) {
        List<int> list = List.empty(growable: true);
        for (var i = 0; i < 100000; i++) {
          if (it.moveNext()) {
            list.add(it.current);
          }
        }
        if (list.isEmpty) break;
        yield f(list);
      }
    }
  }
}
