import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import '../../../hera/hera.dart';
import '../profile_avatar/profile_avatar.dart';

class ProfileCard extends StatelessWidget {
  ProfileCard({
    Key? key,
    this.width,
    this.profileCardDecoration,
    this.onClick,
  }) : super(key: key);

  final double? width;
  final BoxDecoration? profileCardDecoration;
  final Function? onClick;

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: width,
      height: 150,
      child: Container(
        padding: EdgeInsets.only(left: 10, top: 10, bottom: 7.5, right: 7.5),
        decoration: Hera.currentUser().profile.coverImage.url.isNotEmpty ? BoxDecoration(
          image: DecorationImage(
            image: NetworkImage(Hera.currentUser().profile.coverImage.url),
            fit: BoxFit.fill,
          ),
          borderRadius: BorderRadius.all(
            Radius.circular(10),
          ),
        ):
        BoxDecoration(
          color: CupertinoColors.black,
          borderRadius: BorderRadius.all(
            Radius.circular(10),
          ),
        ),
        child: Row(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.start,
          children: [
            ProfileAvatar(
              radius: 40,
              borderColor: CupertinoColors.white.withOpacity(0.5),
              borderRadius: 2.5,
            ),
            SizedBox(width: 20),
            Column(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                if (Hera.currentUser().profile.firstName.isNotEmpty &&
                    Hera.currentUser().profile.lastName.isNotEmpty)
                  Padding(
                    padding: const EdgeInsets.only(bottom: 5.0),
                    child: Text(
                      "${Hera.currentUser().profile.firstName} ${Hera.currentUser().profile.lastName}",
                      style: Theme.of(context).textTheme.titleMedium?.copyWith(
                            color: CupertinoColors.white,
                            fontWeight: FontWeight.w500,
                            fontSize: 22,
                          ),
                      overflow: TextOverflow.ellipsis,
                      textAlign: TextAlign.left,
                      maxLines: 2,
                    ),
                  ),
                if ((Hera.currentUser().profile.firstName.isEmpty ||
                    Hera.currentUser().profile.lastName.isEmpty) && Hera.currentUser().username.isNotEmpty)
                  Padding(
                    padding: const EdgeInsets.only(bottom: 5.0),
                    child: Text(
                      "${Hera.currentUser().username}",
                      style: Theme.of(context).textTheme.titleMedium?.copyWith(
                        color: CupertinoColors.white,
                        fontWeight: FontWeight.w500,
                        fontSize: 22,
                      ),
                      overflow: TextOverflow.ellipsis,
                      textAlign: TextAlign.left,
                      maxLines: 2,
                    ),
                  ),
                if (Hera.currentUser().contactInfo.email.isNotEmpty)
                  Padding(
                    padding: const EdgeInsets.only(bottom: 5.0),
                    child: Text(
                      "${Hera.currentUser().contactInfo.email}",
                      style: Theme.of(context).textTheme.titleMedium?.copyWith(
                            color: CupertinoColors.white,
                            fontWeight: FontWeight.w500,
                        fontSize: 15,
                          ),
                      overflow: TextOverflow.ellipsis,
                      textAlign: TextAlign.left,
                      maxLines: 2,
                    ),
                  ),
                if (Hera.currentUser().contactInfo.phone.isNotEmpty)
                  Padding(
                    padding: const EdgeInsets.only(bottom: 5.0),
                    child: Text(
                      "${Hera.currentUser().contactInfo.phone}",
                      style: Theme.of(context).textTheme.titleMedium?.copyWith(
                        color: CupertinoColors.white,
                        fontWeight: FontWeight.w500,
                        fontSize: 15,
                      ),
                      overflow: TextOverflow.ellipsis,
                      textAlign: TextAlign.left,
                      maxLines: 2,
                    ),
                  ),
              ],
            ),
            Spacer(),
          ],
        ),
      ),
    );
  }
}
