import 'dart:async';
import 'package:flutter/material.dart';
import 'package:nuntio_hera/hera_app/helpers.dart';
import 'package:nuntio_hera/hera_app/hera_localizations.dart';
import 'package:nuntio_hera/hera_app/verification_screen.dart';
import 'package:nuntio_core/components/iconed_button.dart';
import 'package:nuntio_core/components/nuntio_state_button.dart';
import 'package:nuntio_core/components/nuntio_text_field.dart';
import 'package:nuntio_core/models/hera.pb.dart';
import '../hera/hera.dart';
import 'package:nuntio_core/models/nuntio.pb.dart';
import 'package:nuntio_core/components/whatever_image.dart';
import 'hera_icons.dart';

class RegisterScreen extends StatefulWidget {
  static const id = "register";
  const RegisterScreen(this.loginType);

  // config
  final LoginType loginType;

  @override
  State<RegisterScreen> createState() => _RegisterScreenState();
}

class _RegisterScreenState extends State<RegisterScreen> {
  // controllers
  final TextEditingController _identifierController = TextEditingController();
  final TextEditingController _passwordController = TextEditingController();
  final TextEditingController _repeatController = TextEditingController();

  ButtonState _buttonState = ButtonState.idle;

  final _formKey = GlobalKey<FormState>();

  register(BuildContext context) {
    if (_formKey.currentState?.validate() == true) {
      setState(() {
        _buttonState = ButtonState.loading;
      });
      UserIdentifier _identifier = UserIdentifier(
        email: widget.loginType == LoginType.EMAIL_PASSWORD
            ? _identifierController.text
            : "",
        phone: widget.loginType == LoginType.PHONE_PASSWORD
            ? _identifierController.text
            : "",
        username: widget.loginType == LoginType.USERNAME_PASSWORD
            ? _identifierController.text
            : "",
      );
      Hera.createUser(
        identifier: _identifier,
        password: _passwordController.text,
      ).then((value) {
        setState(() {
          _buttonState = ButtonState.success;
        });
        Timer(Duration(seconds: 1), () {
          Hera.login(
                  identifier: _identifier, password: _passwordController.text)
              .then((value) {
            if (value.loginStatus == LoginStatus.EMAIL_IS_NOT_VERIFIED) {
              Navigator.of(context)
                  .pushNamed(
                  VerificationScreen.id,
                  arguments: VerificationScreenArguments(
                      _identifierController.text,
                      null,
                      _passwordController.text,
                      value.expiresAt.toDateTime(),
                      LoginType.EMAIL_PASSWORD))
                  .then((value) => Navigator.of(context).pop());
            } else if (value.loginStatus == LoginStatus.PHONE_IS_NOT_VERIFIED) {
              Navigator.of(context)
                  .pushNamed(
                  VerificationScreen.id,
                  arguments: VerificationScreenArguments(
                      null,
                      _identifierController.text,
                      _passwordController.text,
                      value.expiresAt.toDateTime(),
                      LoginType.PHONE_PASSWORD))
                  .then((value) => Navigator.of(context).pop());
            } else {
              Navigator.of(context).pop();
            }
          }).catchError((_) {
            Navigator.of(context).pop();
          });
        });
      }).catchError((err) {
        setState(() {
          _buttonState = ButtonState.fail;
        });
        Timer(Duration(seconds: 2), () {
          setState(() {
            _buttonState = ButtonState.idle;
          });
        });
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SingleChildScrollView(
        child: Center(
          child: Container(
            height: MediaQuery.of(context).size.height,
            constraints: BoxConstraints(
                minHeight: MediaQuery.of(context).size.height, maxWidth: 500),
            child: SafeArea(
              child: Padding(
                padding: EdgeInsets.only(left: 20, right: 20, bottom: 20),
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    WhatEverImage(Hera.config.logo.url, 100),
                    Text(
                      HeraLocalizations.of(context).registerTitle,
                      style: Theme.of(context).textTheme.displayMedium,
                    ),
                    SizedBox(height: 10),
                    Text(
                      HeraLocalizations.of(context).registerSubtitle,
                      style: Theme.of(context).textTheme.headlineSmall,
                      textAlign: TextAlign.center,
                    ),
                    SizedBox(height: 30),
                    Form(
                      key: _formKey,
                      child: Column(
                        children: [
                          if (widget.loginType == LoginType.EMAIL_PASSWORD)
                            NuntioTextField(
                              prefix: HeraIcons.instance.email,
                              label: HeraLocalizations.of(context).emailLabel,
                              hint: HeraLocalizations.of(context).emailHint,
                              controller: _identifierController,
                              textInputType: TextInputType.emailAddress,
                              textInputAction: TextInputAction.next,
                              validator: (text) {
                                if (text == null ||
                                    text.isEmpty ||
                                    validateEmail(text) == false) {
                                  return HeraLocalizations.of(context)
                                      .invalidEmailError;
                                }
                                return null;
                              },
                            ),
                          if (widget.loginType == LoginType.PHONE_PASSWORD)
                            NuntioTextField(
                              prefix: HeraIcons.instance.phone,
                              label: HeraLocalizations.of(context).phoneLabel,
                              hint: HeraLocalizations.of(context).phoneHint,
                              controller: _identifierController,
                              textInputType: TextInputType.phone,
                              textInputAction: TextInputAction.next,
                              validator: (text) {
                                if (text == null ||
                                    text.isEmpty ||
                                    validateMobile(text) == false) {
                                  return HeraLocalizations.of(context)
                                      .invalidPhoneError;
                                }
                                return null;
                              },
                            ),
                          if (widget.loginType == LoginType.USERNAME_PASSWORD)
                            NuntioTextField(
                              prefix: HeraIcons.instance.username,
                              label:
                                  HeraLocalizations.of(context).usernameLabel,
                              hint: HeraLocalizations.of(context).usernameHint,
                              controller: _identifierController,
                              textInputType: TextInputType.text,
                              textInputAction: TextInputAction.next,
                              validator: (text) {
                                if (text == null ||
                                    text.isEmpty ||
                                    validateUsername(text) == false) {
                                  return HeraLocalizations.of(context)
                                      .invalidUsernameError;
                                }
                                return null;
                              },
                            ),
                          SizedBox(height: 10),
                          NuntioTextField(
                            prefix: HeraIcons.instance.password,
                            label: HeraLocalizations.of(context).passwordLabel,
                            hint: HeraLocalizations.of(context).passwordHint,
                            controller: _passwordController,
                            textInputType: TextInputType.text,
                            textInputAction: TextInputAction.next,
                            obscureText: true,
                            validator: (text) {
                              if (!Hera.config.validatePassword) return null;
                              if (text == null ||
                                  text.isEmpty ||
                                  !_passwordController.text
                                      .contains(RegExp(r'[0-9]'))) {
                                return HeraLocalizations.of(context)
                                    .passwordMustContainNumberError;
                              } else if (!_passwordController.text.contains(
                                  RegExp(r'[!@#$%^&*(),.?":{}|<>]'))) {
                                return HeraLocalizations.of(context)
                                    .passwordMustContainSpecialCharError;
                              } else if (_passwordController.text.length < 8) {
                                return HeraLocalizations.of(context)
                                    .passwordMustMin8LengthError;
                              }
                              return null;
                            },
                          ),
                          SizedBox(height: 10),
                          NuntioTextField(
                            prefix: HeraIcons.instance.repeatPassword,
                            label: HeraLocalizations.of(context)
                                .repeatPasswordLabel,
                            hint: HeraLocalizations.of(context)
                                .repeatPasswordHint,
                            controller: _repeatController,
                            textInputType: TextInputType.text,
                            textInputAction: TextInputAction.done,
                            obscureText: true,
                            validator: (_) {
                              if (_passwordController.text !=
                                  _repeatController.text) {
                                return HeraLocalizations.of(context)
                                    .passwordDoNotMatchError;
                              }
                              return null;
                            },
                          ),
                        ],
                      ),
                    ),
                    SizedBox(height: 20),
                    SizedBox(
                      width: double.infinity,
                      height: Theme.of(context).buttonTheme.height,
                      child: NuntioStateButton.icon(
                        context,
                        iconedButtons: {
                          ButtonState.idle: IconedButton(
                            text: HeraLocalizations.of(context).registerButton,
                            icon: Icon(
                              HeraIcons.instance.login,
                            ),
                            color: Theme.of(context).primaryColor,
                          ),
                          ButtonState.loading: IconedButton(
                            text: "",
                            color: Theme.of(context).primaryColor,
                          ),
                          ButtonState.fail: IconedButton(
                            text: HeraLocalizations.of(context).failure,
                            icon: Icon(Icons.cancel),
                            color: Colors.red.shade300,
                          ),
                          ButtonState.success: IconedButton(
                            text: HeraLocalizations.of(context).success,
                            icon: Icon(Icons.check_circle),
                            color: Colors.green.shade400,
                          )
                        },
                        onPressed: () => register(context),
                        state: _buttonState,
                      ),
                    ),
                    SizedBox(height: 10),
                    TextButton(
                      child: Text(
                        HeraLocalizations.of(context).alreadyHaveAnAccount,
                        style: TextStyle(
                          color: Theme.of(context).primaryColor,
                        ),
                      ),
                      onPressed: () => Navigator.of(context).pop(),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
